//
//  ChimpanzeeGameAppDelegate.m
//  ChimpanzeeGame
//
//  Created by kunii on 08/10/13.
//  Copyright __MyCompanyName__ 2008. All rights reserved.
//

#import "ChimpanzeeGameViewController.h"
#import "ChimpanzeeExaminer.h"

static const int SEQUENCE_TERMINATOR = -1;	//	clickSquence[index]の値が、この値の場合終了。

@implementation ChimpanzeeGameViewController

/*
	なぜかhiddenはアニメーションに対応してないような...
	しかたないのでhiddenを使わず、ボタンを大きくしながらα値を落とすことでフェードアウトする。
	alphaが0.0の場合hideen = TRUEと同じ効果があり、ボタンの配置場所をクリックしてもなにも反応しない。
*/
static void hideButton(
	UIButton*	inButton,
	double		inDelay)
{
	CGAffineTransform scaling = CGAffineTransformMakeScale(1.0, 1.0);
	inButton.transform = scaling;
	inButton.alpha = 1.0;
	CGAffineTransform scaling2 = CGAffineTransformMakeScale(2.5, 2.5);
	[UIView beginAnimations:nil context:NULL];
	[UIView setAnimationDuration:1.0];
	[UIView setAnimationDelay:inDelay];
	inButton.transform = scaling2;
	inButton.alpha = 0.0;
	inButton.enabled = FALSE;
	[UIView commitAnimations];
}

static void blinkButton(
	UIButton*	inButton)
{
	CGAffineTransform scaling = CGAffineTransformMakeScale(1.0, 1.0);
	inButton.transform = scaling;
	inButton.alpha = 1;
	CGAffineTransform scaling2 = CGAffineTransformMakeScale(1.5, 1.5);
	[UIView beginAnimations:nil context:NULL];
	[UIView setAnimationDuration:1];
	[UIView setAnimationRepeatCount:8000];
//	[UIView setAnimationRepeatAutoreverses:TRUE];
	inButton.transform = scaling2;
	inButton.alpha = 0.1;
	[UIView commitAnimations];
}

/*
	ボタンクリック対応
*/
- (IBAction)respondToButtonClick:(UIButton*)sender 
{
    NSInteger tag = sender.tag;		//	タグ取りだし
	if ([examiner judge:tag]) {
		hideButton(sender, 0.0);
		index++;					//	次の順序に進める。
	} else {
		[self start:NSLocalizedString(@"Failed.", @"間違い")];//	間違いというアラートを表示してゲーム再開
		return;
	}
	if (index == level) {
		if (level < 9)
			level++;
		[self start:NSLocalizedString(@"Complite!",@"成功")];	//	成功というアラートを表示してゲーム再開
	}
}


/*
 Implement loadView if you want to create a view hierarchy programmatically
- (void)loadView {
}
 */

/*
 Implement viewDidLoad if you need to do additional setup after loading the view.
*/
- (void)viewDidLoad 
{
	srand(time(NULL));
//	level = 5;
	examiner = [[ChimpanzeeExaminer alloc] init];
	static const int BUTTON_WIDTH = 100;	//	ボタンの幅
	static const int BUTTON_HEIGHT = 100;	//	ボタンの高さ
	static const int BUTTON_MERGIN = 10;	//	ボタン間隔
	[super viewDidLoad];
	UIImage *buttonImage = [UIImage imageNamed:@"button.png"];
	int i = 0;
	for (int v = 0; v < BUTTON_V_COUNT; v++) { 
		for (int h = 0; h < BUTTON_H_COUNT; h++) { 
			CGRect frame = CGRectMake(			//	ボタン位置設定
				BUTTON_MERGIN + h * (BUTTON_WIDTH + BUTTON_MERGIN), 
				BUTTON_MERGIN + v * (BUTTON_HEIGHT + BUTTON_MERGIN), 
				BUTTON_WIDTH, BUTTON_HEIGHT);
			UIButton *bt = [[UIButton alloc] initWithFrame:frame];
			[bt setBackgroundImage:buttonImage forState:UIControlStateNormal];
			bt.contentVerticalAlignment = UIControlContentVerticalAlignmentCenter;
			bt.contentHorizontalAlignment = UIControlContentHorizontalAlignmentCenter;
			[bt addTarget:self action:@selector(respondToButtonClick:)	//	アクション設定
				forControlEvents:UIControlEventTouchUpInside];
			bt.tag = i;						//	タグ設定
			bt.hidden = TRUE;				//	最初は非表示
			bt.enabled = FALSE;				//	最初はボタンを押せない状態にする。
			//	スケール中心をボタンの中心に設定する。
			CGPoint center;
			center.x = bt.frame.origin.x + bt.frame.size.width / 2;
			center.y = bt.frame.origin.y + bt.frame.size.height / 2;
			bt.center = center;
			[self.view addSubview:bt];			//	ビューに登録
			buttons[i++] = bt;				//	自身の変数に登録
		}
	}
	[self start:@""];						//	開始
 }

/*
	全ボタンを隠す
*/
- (void)hideAllButtons
{
	int i = 0;
	for (int v = 0; v < BUTTON_V_COUNT; v++) { 
		for (int h = 0; h < BUTTON_H_COUNT; h++) { 
			buttons[i++].hidden = TRUE;
		}
	}
}

/*
	messageをアラートで表示してゲーム開始。
*/
- (void)start:(NSString*)message
{
	[self hideAllButtons];					//	全ボタンを隠す
	const int* locs = [examiner startGame:level]; 
	for (int i = 0; i < level; i++) {
		clickSquence[i] = locs[i];
	}
	clickSquence[level] = SEQUENCE_TERMINATOR;
	index = 0;								//	チェックする順序を先頭にする。
	//	開始画面表示
	UIAlertView *alert = [[UIAlertView alloc] initWithTitle: NSLocalizedString(@"chimpanzee!", @"アラートのタイトル")
		message:message
		delegate:self cancelButtonTitle:NSLocalizedString(@"Start", @"開始") otherButtonTitles: nil];
	[alert show];	
	[alert release];	
}

/*
	アラートクリック後の処理
	
	クリックする順番を、1〜（人の感覚に合うよう、0ではなく1から始める）としてボタン自身に表示させる。
*/
- (void)alertView:(UIAlertView *)actionSheet clickedButtonAtIndex:(NSInteger)buttonIndex
{
	static const double SHOW_SEQUENCE_TIME = 2.0;	//	クリック順を見せている時間。
	for (int i = 0 ; clickSquence[i] != SEQUENCE_TERMINATOR; i++) {
		UIButton* bt = buttons[clickSquence[i]];
		[bt setTitle:[NSString stringWithFormat:@"%d", i + 1] forState:UIControlStateNormal];
		bt.enabled = FALSE;		//	ただし、押されて反応させないようにする。
		bt.hidden = FALSE;		//	表示
		hideButton(bt, 0.5 * i);
	}
	//	タイマーでSHOW_SEQUENCE_TIME秒後にボタンを消えるようにする
	[[NSTimer scheduledTimerWithTimeInterval:SHOW_SEQUENCE_TIME
		target:self selector:@selector(hideClickSequence:) userInfo:0 repeats:NO] retain];
}

/*
	いったん全ボタンを消し、0.5秒後にボタンの順序を隠した状態で表示。
*/
- (void)hideClickSequence:( NSTimer *)aTimer
{
	static const double BLACKOUT_TIME = 0.5;	//	全ボタンが消えている時間。
	[self hideAllButtons];						//	全ボタンを隠す。見えているのは一部だが...
	[aTimer release];							//	タイマー開放
	//	新しいタイマーでBLACKOUT_TIME秒後にボタンを消えるようにする
	[[NSTimer scheduledTimerWithTimeInterval:BLACKOUT_TIME
		target:self selector:@selector(startGame:) userInfo:0 repeats:NO] retain];
}

/*
	ゲーム開始
*/
- (void)startGame:( NSTimer *)aTimer
{
	for (int i = 0 ; clickSquence[i] != SEQUENCE_TERMINATOR; i++) {
		UIButton* bt = buttons[clickSquence[i]];
		[bt setTitle:@"" forState:UIControlStateNormal];	//	タイトルは空にする。
		bt.hidden = FALSE;						//	表示
		bt.enabled = TRUE;						//	今回はクリックを受け付ける。
		[UIView commitAnimations];
		blinkButton(bt);
	}
	[aTimer release];							//	タイマー開放
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
	// Return YES for supported orientations
	return (interfaceOrientation == UIInterfaceOrientationPortrait);
}


- (void)didReceiveMemoryWarning {
	[super didReceiveMemoryWarning]; // Releases the view if it doesn't have a superview
	// Release anything that's not essential, such as cached data
}

/*
	終了
	利用していたボタンを開放する。
*/
- (void)dealloc {
	int i = 0;	//	ボタンインディックス
	for (int v = 0; v < BUTTON_V_COUNT; v++) { 
		for (int h = 0; h < BUTTON_H_COUNT; h++) {
			[buttons[i++] release];	//	開放
		}
	}
	[examiner dealloc];
	[super dealloc];
}

- (void)setLevel:(NSInteger)inLevel
{
	level = inLevel;
}

- (int)getLevel
{
	return level;
}

@end
/*
	CGAffineTransform form = CGAffineTransformMakeRotation(0.0);
		buttons[clickSquence[n]].transform = form;
*/